from enum import StrEnum


class StandardText(StrEnum):
    """Standard text constants."""

    POST = "#POST"
    PLAYLIST = "#PLAYLIST"
    LEVEL = "#LEVEL"
    SKIN = "#SKIN"
    BACKGROUND = "#BACKGROUND"
    EFFECT = "#EFFECT"
    PARTICLE = "#PARTICLE"
    ENGINE = "#ENGINE"
    REPLAY = "#REPLAY"
    ROOM = "#ROOM"
    GRADE = "#GRADE"
    ARCADE_SCORE = "#ARCADE_SCORE"
    ACCURACY_SCORE = "#ACCURACY_SCORE"
    COMBO = "#COMBO"
    PERFECT = "#PERFECT"
    GREAT = "#GREAT"
    GOOD = "#GOOD"
    MISS = "#MISS"
    KEYWORDS = "#KEYWORDS"
    NAME = "#NAME"
    RATING = "#RATING"
    RATING_MINIMUM = "#RATING_MINIMUM"
    RATING_MAXIMUM = "#RATING_MAXIMUM"
    TITLE = "#TITLE"
    SUBTITLE = "#SUBTITLE"
    ARTISTS = "#ARTISTS"
    TIME = "#TIME"
    AUTHOR = "#AUTHOR"
    GENRE = "#GENRE"
    TYPE = "#TYPE"
    CATEGORY = "#CATEGORY"
    LANGUAGE = "#LANGUAGE"
    DIFFICULTY = "#DIFFICULTY"
    VERSION = "#VERSION"
    LENGTH = "#LENGTH"
    LENGTH_MINIMUM = "#LENGTH_MINIMUM"
    LENGTH_MAXIMUM = "#LENGTH_MAXIMUM"
    ADDITIONAL_INFORMATION = "#ADDITIONAL_INFORMATION"
    TIMEZONE = "#TIMEZONE"
    REGION = "#REGION"
    CONTENT = "#CONTENT"
    COMMENT = "#COMMENT"
    REVIEW = "#REVIEW"
    REPLY = "#REPLY"
    MESSAGE = "#MESSAGE"
    ROLE = "#ROLE"
    PERMISSION = "#PERMISSION"
    USER = "#USER"
    SPEED = "#SPEED"
    MIRROR = "#MIRROR"
    RANDOM = "#RANDOM"
    HIDDEN = "#HIDDEN"
    JUDGMENT_STRICT = "#JUDGMENT_STRICT"
    JUDGMENT_LOOSE = "#JUDGMENT_LOOSE"
    EFFECT_AUTO = "#EFFECT_AUTO"
    STAGE = "#STAGE"
    STAGE_POSITION = "#STAGE_POSITION"
    STAGE_SIZE = "#STAGE_SIZE"
    STAGE_ROTATION = "#STAGE_ROTATION"
    STAGE_DIRECTION = "#STAGE_DIRECTION"
    STAGE_ALPHA = "#STAGE_ALPHA"
    STAGE_ANIMATION = "#STAGE_ANIMATION"
    STAGE_TILT = "#STAGE_TILT"
    STAGE_COVER_VERTICAL = "#STAGE_COVER_VERTICAL"
    STAGE_COVER_HORIZONTAL = "#STAGE_COVER_HORIZONTAL"
    STAGE_COVER_ALPHA = "#STAGE_COVER_ALPHA"
    STAGE_ASPECTRATIO_LOCK = "#STAGE_ASPECTRATIO_LOCK"
    STAGE_EFFECT = "#STAGE_EFFECT"
    STAGE_EFFECT_POSITION = "#STAGE_EFFECT_POSITION"
    STAGE_EFFECT_SIZE = "#STAGE_EFFECT_SIZE"
    STAGE_EFFECT_ROTATION = "#STAGE_EFFECT_ROTATION"
    STAGE_EFFECT_DIRECTION = "#STAGE_EFFECT_DIRECTION"
    STAGE_EFFECT_ALPHA = "#STAGE_EFFECT_ALPHA"
    LANE = "#LANE"
    LANE_POSITION = "#LANE_POSITION"
    LANE_SIZE = "#LANE_SIZE"
    LANE_ROTATION = "#LANE_ROTATION"
    LANE_DIRECTION = "#LANE_DIRECTION"
    LANE_ALPHA = "#LANE_ALPHA"
    LANE_ANIMATION = "#LANE_ANIMATION"
    LANE_EFFECT = "#LANE_EFFECT"
    LANE_EFFECT_POSITION = "#LANE_EFFECT_POSITION"
    LANE_EFFECT_SIZE = "#LANE_EFFECT_SIZE"
    LANE_EFFECT_ROTATION = "#LANE_EFFECT_ROTATION"
    LANE_EFFECT_DIRECTION = "#LANE_EFFECT_DIRECTION"
    LANE_EFFECT_ALPHA = "#LANE_EFFECT_ALPHA"
    JUDGELINE = "#JUDGELINE"
    JUDGELINE_POSITION = "#JUDGELINE_POSITION"
    JUDGELINE_SIZE = "#JUDGELINE_SIZE"
    JUDGELINE_ROTATION = "#JUDGELINE_ROTATION"
    JUDGELINE_DIRECTION = "#JUDGELINE_DIRECTION"
    JUDGELINE_ALPHA = "#JUDGELINE_ALPHA"
    JUDGELINE_ANIMATION = "#JUDGELINE_ANIMATION"
    JUDGELINE_EFFECT = "#JUDGELINE_EFFECT"
    JUDGELINE_EFFECT_POSITION = "#JUDGELINE_EFFECT_POSITION"
    JUDGELINE_EFFECT_SIZE = "#JUDGELINE_EFFECT_SIZE"
    JUDGELINE_EFFECT_ROTATION = "#JUDGELINE_EFFECT_ROTATION"
    JUDGELINE_EFFECT_DIRECTION = "#JUDGELINE_EFFECT_DIRECTION"
    JUDGELINE_EFFECT_ALPHA = "#JUDGELINE_EFFECT_ALPHA"
    SLOT = "#SLOT"
    SLOT_POSITION = "#SLOT_POSITION"
    SLOT_SIZE = "#SLOT_SIZE"
    SLOT_ROTATION = "#SLOT_ROTATION"
    SLOT_DIRECTION = "#SLOT_DIRECTION"
    SLOT_ALPHA = "#SLOT_ALPHA"
    SLOT_ANIMATION = "#SLOT_ANIMATION"
    SLOT_EFFECT = "#SLOT_EFFECT"
    SLOT_EFFECT_POSITION = "#SLOT_EFFECT_POSITION"
    SLOT_EFFECT_SIZE = "#SLOT_EFFECT_SIZE"
    SLOT_EFFECT_ROTATION = "#SLOT_EFFECT_ROTATION"
    SLOT_EFFECT_DIRECTION = "#SLOT_EFFECT_DIRECTION"
    SLOT_EFFECT_ALPHA = "#SLOT_EFFECT_ALPHA"
    NOTE = "#NOTE"
    NOTE_SPEED = "#NOTE_SPEED"
    NOTE_SPEED_RANDOM = "#NOTE_SPEED_RANDOM"
    NOTE_POSITION = "#NOTE_POSITION"
    NOTE_SIZE = "#NOTE_SIZE"
    NOTE_ROTATION = "#NOTE_ROTATION"
    NOTE_DIRECTION = "#NOTE_DIRECTION"
    NOTE_COLOR = "#NOTE_COLOR"
    NOTE_ALPHA = "#NOTE_ALPHA"
    NOTE_ANIMATION = "#NOTE_ANIMATION"
    NOTE_EFFECT = "#NOTE_EFFECT"
    NOTE_EFFECT_POSITION = "#NOTE_EFFECT_POSITION"
    NOTE_EFFECT_SIZE = "#NOTE_EFFECT_SIZE"
    NOTE_EFFECT_ROTATION = "#NOTE_EFFECT_ROTATION"
    NOTE_EFFECT_DIRECTION = "#NOTE_EFFECT_DIRECTION"
    NOTE_EFFECT_COLOR = "#NOTE_EFFECT_COLOR"
    NOTE_EFFECT_ALPHA = "#NOTE_EFFECT_ALPHA"
    MARKER = "#MARKER"
    MARKER_POSITION = "#MARKER_POSITION"
    MARKER_SIZE = "#MARKER_SIZE"
    MARKER_ROTATION = "#MARKER_ROTATION"
    MARKER_DIRECTION = "#MARKER_DIRECTION"
    MARKER_COLOR = "#MARKER_COLOR"
    MARKER_ALPHA = "#MARKER_ALPHA"
    MARKER_ANIMATION = "#MARKER_ANIMATION"
    CONNECTOR = "#CONNECTOR"
    CONNECTOR_POSITION = "#CONNECTOR_POSITION"
    CONNECTOR_SIZE = "#CONNECTOR_SIZE"
    CONNECTOR_ROTATION = "#CONNECTOR_ROTATION"
    CONNECTOR_DIRECTION = "#CONNECTOR_DIRECTION"
    CONNECTOR_COLOR = "#CONNECTOR_COLOR"
    CONNECTOR_ALPHA = "#CONNECTOR_ALPHA"
    CONNECTOR_ANIMATION = "#CONNECTOR_ANIMATION"
    SIMLINE = "#SIMLINE"
    SIMLINE_POSITION = "#SIMLINE_POSITION"
    SIMLINE_SIZE = "#SIMLINE_SIZE"
    SIMLINE_ROTATION = "#SIMLINE_ROTATION"
    SIMLINE_DIRECTION = "#SIMLINE_DIRECTION"
    SIMLINE_COLOR = "#SIMLINE_COLOR"
    SIMLINE_ALPHA = "#SIMLINE_ALPHA"
    SIMLINE_ANIMATION = "#SIMLINE_ANIMATION"
    PREVIEW_SCALE_VERTICAL = "#PREVIEW_SCALE_VERTICAL"
    PREVIEW_SCALE_HORIZONTAL = "#PREVIEW_SCALE_HORIZONTAL"
    PREVIEW_TIME = "#PREVIEW_TIME"
    PREVIEW_SCORE = "#PREVIEW_SCORE"
    PREVIEW_BPM = "#PREVIEW_BPM"
    PREVIEW_TIMESCALE = "#PREVIEW_TIMESCALE"
    PREVIEW_BEAT = "#PREVIEW_BEAT"
    PREVIEW_MEASURE = "#PREVIEW_MEASURE"
    PREVIEW_COMBO = "#PREVIEW_COMBO"
    NONE = "#NONE"
    ANY = "#ANY"
    ALL = "#ALL"
    OTHERS = "#OTHERS"
    SHORT = "#SHORT"
    LONG = "#LONG"
    HIGH = "#HIGH"
    MID = "#MID"
    LOW = "#LOW"
    SMALL = "#SMALL"
    MEDIUM = "#MEDIUM"
    LARGE = "#LARGE"
    LEFT = "#LEFT"
    RIGHT = "#RIGHT"
    UP = "#UP"
    DOWN = "#DOWN"
    FRONT = "#FRONT"
    BACK = "#BACK"
    CENTER = "#CENTER"
    TOP = "#TOP"
    BOTTOM = "#BOTTOM"
    TOP_LEFT = "#TOP_LEFT"
    TOP_CENTER = "#TOP_CENTER"
    TOP_RIGHT = "#TOP_RIGHT"
    CENTER_LEFT = "#CENTER_LEFT"
    CENTER_RIGHT = "#CENTER_RIGHT"
    BOTTOM_LEFT = "#BOTTOM_LEFT"
    BOTTOM_CENTER = "#BOTTOM_CENTER"
    BOTTOM_RIGHT = "#BOTTOM_RIGHT"
    CLOCKWISE = "#CLOCKWISE"
    COUNTERCLOCKWISE = "#COUNTERCLOCKWISE"
    FORWARD = "#FORWARD"
    BACKWARD = "#BACKWARD"
    DEFAULT = "#DEFAULT"
    NEUTRAL = "#NEUTRAL"
    RED = "#RED"
    GREEN = "#GREEN"
    BLUE = "#BLUE"
    YELLOW = "#YELLOW"
    PURPLE = "#PURPLE"
    CYAN = "#CYAN"
    SIMPLE = "#SIMPLE"
    EASY = "#EASY"
    NORMAL = "#NORMAL"
    HARD = "#HARD"
    EXPERT = "#EXPERT"
    MASTER = "#MASTER"
    PRO = "#PRO"
    TECHNICAL = "#TECHNICAL"
    SPECIAL = "#SPECIAL"
    KEYWORDS_PLACEHOLDER = "#KEYWORDS_PLACEHOLDER"
    NAME_PLACEHOLDER = "#NAME_PLACEHOLDER"
    RATING_PLACEHOLDER = "#RATING_PLACEHOLDER"
    RATING_MINIMUM_PLACEHOLDER = "#RATING_MINIMUM_PLACEHOLDER"
    RATING_MAXIMUM_PLACEHOLDER = "#RATING_MAXIMUM_PLACEHOLDER"
    TITLE_PLACEHOLDER = "#TITLE_PLACEHOLDER"
    SUBTITLE_PLACEHOLDER = "#SUBTITLE_PLACEHOLDER"
    ARTISTS_PLACEHOLDER = "#ARTISTS_PLACEHOLDER"
    TIME_PLACEHOLDER = "#TIME_PLACEHOLDER"
    AUTHOR_PLACEHOLDER = "#AUTHOR_PLACEHOLDER"
    GENRE_PLACEHOLDER = "#GENRE_PLACEHOLDER"
    TYPE_PLACEHOLDER = "#TYPE_PLACEHOLDER"
    CATEGORY_PLACEHOLDER = "#CATEGORY_PLACEHOLDER"
    LANGUAGE_PLACEHOLDER = "#LANGUAGE_PLACEHOLDER"
    DIFFICULTY_PLACEHOLDER = "#DIFFICULTY_PLACEHOLDER"
    LENGTH_PLACEHOLDER = "#LENGTH_PLACEHOLDER"
    LENGTH_MINIMUM_PLACEHOLDER = "#LENGTH_MINIMUM_PLACEHOLDER"
    LENGTH_MAXIMUM_PLACEHOLDER = "#LENGTH_MAXIMUM_PLACEHOLDER"
    ADDITIONAL_INFORMATION_PLACEHOLDER = "#ADDITIONAL_INFORMATION_PLACEHOLDER"
    TIMEZONE_PLACEHOLDER = "#TIMEZONE_PLACEHOLDER"
    REGION_PLACEHOLDER = "#REGION_PLACEHOLDER"
    CONTENT_PLACEHOLDER = "#CONTENT_PLACEHOLDER"
    COMMENT_PLACEHOLDER = "#COMMENT_PLACEHOLDER"
    REVIEW_PLACEHOLDER = "#REVIEW_PLACEHOLDER"
    REPLY_PLACEHOLDER = "#REPLY_PLACEHOLDER"
    MESSAGE_PLACEHOLDER = "#MESSAGE_PLACEHOLDER"
    ROLE_PLACEHOLDER = "#ROLE_PLACEHOLDER"
    PERMISSION_PLACEHOLDER = "#PERMISSION_PLACEHOLDER"
    USER_PLACEHOLDER = "#USER_PLACEHOLDER"
    PERCENTAGE_UNIT = "#PERCENTAGE_UNIT"
    SECOND_UNIT = "#SECOND_UNIT"
    MILLISECOND_UNIT = "#MILLISECOND_UNIT"
    TAP = "#TAP"
    TAP_HOLD = "#TAP_HOLD"
    TAP_RELEASE = "#TAP_RELEASE"
    TAP_FLICK = "#TAP_FLICK"
    TAP_SLIDE = "#TAP_SLIDE"
    HOLD = "#HOLD"
    HOLD_SLIDE = "#HOLD_SLIDE"
    HOLD_FOLLOW = "#HOLD_FOLLOW"
    RELEASE = "#RELEASE"
    FLICK = "#FLICK"
    SLIDE = "#SLIDE"
    SLIDE_FLICK = "#SLIDE_FLICK"
    AVOID = "#AVOID"
    JIGGLE = "#JIGGLE"
    NEWEST = "#NEWEST"
    OLDEST = "#OLDEST"
    RECOMMENDED = "#RECOMMENDED"
    POPULAR = "#POPULAR"
    FEATURED = "#FEATURED"
    COMPETITIVE = "#COMPETITIVE"
    HOLIDAY = "#HOLIDAY"
    LIMITED = "#LIMITED"
    FOLLOWING = "#FOLLOWING"
    ANNOUNCEMENT = "#ANNOUNCEMENT"
    INFORMATION = "#INFORMATION"
    HELP = "#HELP"
    MAINTENANCE = "#MAINTENANCE"
    EVENT = "#EVENT"
    UPDATE = "#UPDATE"
    SEARCH = "#SEARCH"
    ADVANCED = "#ADVANCED"
    RELATED = "#RELATED"
    SAME_AUTHOR = "#SAME_AUTHOR"
    SAME_ARTISTS = "#SAME_ARTISTS"
    SAME_RATING = "#SAME_RATING"
    SAME_CATEGORY = "#SAME_CATEGORY"
    SAME_DIFFICULTY = "#SAME_DIFFICULTY"
    SAME_GENRE = "#SAME_GENRE"
    SAME_VERSION = "#SAME_VERSION"
    OTHER_AUTHORS = "#OTHER_AUTHORS"
    OTHER_ARTISTS = "#OTHER_ARTISTS"
    OTHER_RATINGS = "#OTHER_RATINGS"
    OTHER_CATEGORIES = "#OTHER_CATEGORIES"
    OTHER_DIFFICULTIES = "#OTHER_DIFFICULTIES"
    OTHER_GENRES = "#OTHER_GENRES"
    OTHER_VERSIONS = "#OTHER_VERSIONS"
    DRAFT = "#DRAFT"
    REVIEWING = "#REVIEWING"
    PUBLISHED = "#PUBLISHED"
    VERIFYING = "#VERIFYING"
    VERIFIED = "#VERIFIED"
    REMOVED = "#REMOVED"
    PUBLIC = "#PUBLIC"
    PRIVATE = "#PRIVATE"
    LOCKED = "#LOCKED"
    POP = "#POP"
    ROCK = "#ROCK"
    HIPHOP = "#HIPHOP"
    COUNTRY = "#COUNTRY"
    ELECTRONIC = "#ELECTRONIC"
    METAL = "#METAL"
    CLASSICAL = "#CLASSICAL"
    FOLK = "#FOLK"
    INDIE = "#INDIE"
    ANIME = "#ANIME"
    VOCALOID = "#VOCALOID"
    REMIX = "#REMIX"
    INSTRUMENTAL = "#INSTRUMENTAL"
    SHORT_VERSION = "#SHORT_VERSION"
    LONG_VERSION = "#LONG_VERSION"
    LIVE_VERSION = "#LIVE_VERSION"
    REPORT = "#REPORT"
    REASON = "#REASON"
    ILLEGAL_ACTIVITIES = "#ILLEGAL_ACTIVITIES"
    CHEATING = "#CHEATING"
    AFK = "#AFK"
    SPAMMING = "#SPAMMING"
    VERBAL_ABUSE = "#VERBAL_ABUSE"
    INAPPROPRIATE_LANGUAGE = "#INAPPROPRIATE_LANGUAGE"
    NEGATIVE_ATTITUDE = "#NEGATIVE_ATTITUDE"
    DNF = "#DNF"
    SUGGESTIONS = "#SUGGESTIONS"
    SUGGESTIONS_PER_PLAYER = "#SUGGESTIONS_PER_PLAYER"
    MATCH_SCORING = "#MATCH_SCORING"
    MATCH_TIEBREAKER = "#MATCH_TIEBREAKER"
    MATCH_COUNT = "#MATCH_COUNT"
    MATCH_LIMIT = "#MATCH_LIMIT"
    ROUND_SCORING = "#ROUND_SCORING"
    ROUND_TIEBREAKER = "#ROUND_TIEBREAKER"
    ROUND_COUNT = "#ROUND_COUNT"
    ROUND_LIMIT = "#ROUND_LIMIT"
    TEAM_SCORING = "#TEAM_SCORING"
    TEAM_TIEBREAKER = "#TEAM_TIEBREAKER"
    TEAM_COUNT = "#TEAM_COUNT"
    TEAM_LIMIT = "#TEAM_LIMIT"
    QUALIFIED = "#QUALIFIED"
    DISQUALIFIED = "#DISQUALIFIED"
    RANKING = "#RANKING"
    SCORE = "#SCORE"
    PLAYER = "#PLAYER"
    SPECTATOR = "#SPECTATOR"
    REFEREE = "#REFEREE"
    ELIMINATED = "#ELIMINATED"
    FINALIST = "#FINALIST"
    FINISHED = "#FINISHED"
    WINNER = "#WINNER"
    GOLD_MEDAL = "#GOLD_MEDAL"
    SILVER_MEDAL = "#SILVER_MEDAL"
    BRONZE_MEDAL = "#BRONZE_MEDAL"
    TEAM_1 = "#TEAM_1"
    TEAM_2 = "#TEAM_2"
    TEAM_3 = "#TEAM_3"
    TEAM_4 = "#TEAM_4"
    TEAM_5 = "#TEAM_5"
    TEAM_6 = "#TEAM_6"
    TEAM_7 = "#TEAM_7"
    TEAM_8 = "#TEAM_8"
    TEAM_RED = "#TEAM_RED"
    TEAM_GREEN = "#TEAM_GREEN"
    TEAM_BLUE = "#TEAM_BLUE"
    TEAM_YELLOW = "#TEAM_YELLOW"
    TEAM_PURPLE = "#TEAM_PURPLE"
    TEAM_CYAN = "#TEAM_CYAN"
    TEAM_WHITE = "#TEAM_WHITE"
    TEAM_BLACK = "#TEAM_BLACK"
    UPLOAD = "#UPLOAD"
    SUBMIT = "#SUBMIT"
    EDIT = "#EDIT"
    LIKE = "#LIKE"
    LIKED = "#LIKED"
    DISLIKE = "#DISLIKE"
    DISLIKED = "#DISLIKED"
    BOOKMARK = "#BOOKMARK"
    BOOKMARKED = "#BOOKMARKED"
    DELETE = "#DELETE"
    RESTORE = "#RESTORE"
    CONFIRM = "#CONFIRM"
    CANCEL = "#CANCEL"
    INCREASE = "#INCREASE"
    DECREASE = "#DECREASE"
    UPVOTE = "#UPVOTE"
    UPVOTED = "#UPVOTED"
    DOWNVOTE = "#DOWNVOTE"
    DOWNVOTED = "#DOWNVOTED"
    AGREE = "#AGREE"
    AGREED = "#AGREED"
    DISAGREE = "#DISAGREE"
    DISAGREED = "#DISAGREED"
    LOCK = "#LOCK"
    UNLOCK = "#UNLOCK"
    PIN = "#PIN"
    UNPIN = "#UNPIN"
    FOLLOW = "#FOLLOW"
    UNFOLLOW = "#UNFOLLOW"
    SUBSCRIBE = "#SUBSCRIBE"
    UNSUBSCRIBE = "#UNSUBSCRIBE"
    PUBLISH = "#PUBLISH"
    UNPUBLISH = "#UNPUBLISH"
    SHOW = "#SHOW"
    HIDE = "#HIDE"
    ALLOW = "#ALLOW"
    DISALLOW = "#DISALLOW"
    APPROVE = "#APPROVE"
    DENY = "#DENY"
    ACCEPT = "#ACCEPT"
    REJECT = "#REJECT"
    STAR = "#STAR"
    STARRED = "#STARRED"
